# opkbuild
# lib/locale
# Locale functions.
#
# Copyright (C) 2012 Patrick "P. J." McDermott
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

[ -n "${_OB_LOCALE_SM}" ] && return 0
_OB_LOCALE_SM='true'

ob_use output

_OB_DEFAULT_LOCALE='en_US'
_OB_INTERNAL_TEXT_DOMAIN='libopkbuild_1'
_OB_TEXT_DOMAIN=
_OB_LOCALE_PATH='@@LOCALEDIR@@/%s/LC_MESSAGES/%s.ms'

ob_get_text_domain()
{
	_ob_local

	if [ ${#} -ne 0 ]; then
		_ob_return 125
		return ${?}
	fi

	echo "${_OB_TEXT_DOMAIN}"
	_ob_return 0
	return ${?}
}

ob_set_text_domain()
{
	_ob_local

	if [ ${#} -eq 1 ]; then
		_OB_TEXT_DOMAIN="${1}"
	else
		_ob_return 125
		return ${?}
	fi

	# Make sure LC_MESSAGES is set.
	if [ -z "${LC_MESSAGES}" ]; then
		if [ -n "${LC_ALL}" ]; then
			LC_MESSAGES="${LC_ALL}"
		elif [ -n "${LANG}" ]; then
			LC_MESSAGES="${LANG}"
		else
			LC_MESSAGES="${_OB_DEFAULT_LOCALE}"
		fi
	fi

	# TODO: Exit early if domain's messages have already been loaded.

	# Try to load the messages.
	if ! _ob_try_load_messages "${LC_MESSAGES%.*}"; then
		if ! _ob_try_load_messages "${LC_MESSAGES%_*}"; then
			if ! _ob_try_load_messages "${_OB_DEFAULT_LOCALE}"; then
				ob_warn 'Cannot load messages in "%s" domain.' \
					"${_OB_TEXT_DOMAIN}"
				_ob_return 1
				return ${?}
			fi
		fi
	fi

	_ob_return 0
	return ${?}
}

ob_set_locale_path()
{
	_ob_local _obslp_format

	if [ ${#} -eq 1 ]; then
		_obslp_format="${1}"
	else
		_ob_return 125
		return ${?}
	fi

	_OB_LOCALE_PATH="${_obslp_format}"

	_ob_return 0
	return ${?}
}

ob_get_msg()
{
	_ob_local _obgm_msgid

	if [ ${#} -eq 1 ]; then
		_obgm_msgid="${1}"
	else
		_ob_return 125
		return ${?}
	fi

	IFS= eval echo \$\{"msg_${_OB_TEXT_DOMAIN}_${_obgm_msgid}"\}

	_ob_return 0
	return ${?}
}

_ob_try_load_messages()
{
	_ob_local _obtlm_locale _obtlm_ms

	_obtlm_locale="${1}"
	_obtlm_ms="$(printf "${_OB_LOCALE_PATH}" \
		"${_obtlm_locale}" "${_OB_TEXT_DOMAIN}")"

	if [ -f "${_obtlm_ms}" ]; then
		. "${_obtlm_ms}"
	else
		_ob_return 1
		return ${?}
	fi

	_ob_return 0
	return ${?}
}
